#!/bin/bash

## ICA: Independente Component Analysis

## This will perform Independent Component Analysis. The output is a series of links which will be used to decide which components are artefacts. Components which contain unwanted artifacts will later be 'rejected' and one should keep a list of these the text file generated by the makereject function on matlab. We will also create a background image which is based off of an MNI image. This will fascilitate visual inspection of the ICAs. 

source preproc2_config.sh

# Preprocessing suffix. Denotes the preprocessing stage of the data.
InputStage="feat_detrended_highpassed" 		# Before generating filtered_func_data.ica

# Test
#SubjectID="EYEMEMtest"

# PBS Log Info
CurrentPreproc="ICA"
CurrentLog="${LogPath}/${CurrentPreproc}"
if [ ! -d ${CurrentLog} ]; then mkdir ${CurrentLog}; chmod 770 ${CurrentLog}; fi

# Error log
Error_Log="${CurrentLog}_error_summary.txt"; echo "" >> ${Error_Log}; chmod 770 ${CurrentLog}

# Loop over participants, sessions (if they exist) & runs/conditions/tasks/etc
for SUB in ${SubjectID} ; do
	if [ -z "${SessionID}" ]; then Session="NoSessions"; SessionFolder=""; SessionName=""
	else Session="${SessionID}"
	fi
	for SES in ${Session}; do
		if [ "${Session}" != "NoSessions" ]; then
			if [ ! -d ${DataPath}/${SUB}/${SES} ]; then continue
			else SessionFolder="${SES}/"; SessionName="${SES}_"
			fi			
		fi
		for RUN in ${RunID}; do
			
			# Name of anatomical and functional images to be used.
			FuncImage="${SUB}_${SessionName}${RUN}"												# Run specific functional image
			## Only S1276 was run with ANTs brain.
			AnatImage="${SUB}_${SessionName}t1_ANTs_brain"											# Brain extracted anatomical image
			# Path to the anatomical and functional image folders.
			AnatPath="${DataPath}/${SUB}/${SessionFolder}mri/t1"					# Path for anatomical image
			FuncPath="${DataPath}/${SUB}/${SessionFolder}${PreprocPipe}/${RUN}"	# Path for run specific functional image
			
			if [ ! -f ${FuncPath}/${FuncImage}_${InputStage}.nii.gz ]; then
				continue
			elif [ -d ${FuncPath}/FEAT.feat/filtered_func_data.ica ]; then
				cd ${FuncPath}/FEAT.feat/filtered_func_data.ica
				Log=`grep "finished!" log.txt | tail -1` # Get line containing our desired text output
				if [ ! "$Log" == "finished!" ]; then
					echo "${SUB} ${RUN}: ICA was incomplete, deleting and re-running"
					rm -rf ${FuncPath}/FEAT.feat/filtered_func_data.ica
					cd ${FuncPath} 		# Job file can't be created in a non-existing folder
				else
					continue
				fi
			fi

			# Gridwise
			echo "#PBS -N ${CurrentPreproc}_${FuncImage}" 				>> job # Job name 
			echo "#PBS -l walltime=4:00:00" 							>> job # Time until job is killed 
			echo "#PBS -l mem=4gb" 										>> job # Books 4gb RAM for the job 
			#echo "#PBS -m n" 											>> job # Email notification on abort/end, use 'n' for no notification 
			echo "#PBS -o ${CurrentLog}" 								>> job # Write (output) log to group log folder 
			echo "#PBS -e ${CurrentLog}" 								>> job # Write (error) log to group log folder 

			# Initialize FSL
			echo ". /etc/fsl/5.0/fsl.sh"								>> job # Set fsl environment 	
			#FSLDIR="/home/mpib/LNDG/FSL/fsl-5.0.11"
			#echo "FSLDIR=${FSLDIR}" 								>> job
			#echo ". ${FSLDIR}/etc/fslconf/fsl.sh"                   >> job
			#echo "PATH=${FSLDIR}/bin:${PATH}"                       >> job
			#echo "export FSLDIR PATH"                               >> job
			
			# Variables for Background Image (Anat2Func)
			a2f_Ref="${FuncPath}/FEAT.feat/reg/example_func.nii.gz" 
			a2f_Init="${FuncPath}/FEAT.feat/reg/highres2example_func.mat" 
			a2f_Out="${FuncPath}/anat2func.nii.gz"										# Background image for ICA
			
			# Variables for ICA
			
			Preproc="${FuncPath}/${FuncImage}_${InputStage}.nii.gz"							# Preprocessed data image
			BET="${AnatPath}/${AnatImage}.nii.gz"											# Brain extracted T1 image
			ICA="${FuncPath}/FEAT.feat/filtered_func_data.ica"								# Location for ICA 
			Report="${FuncPath}/FEAT.feat/filtered_func_data.ica/report.html"				# Location for ICA report links
			
			## Run ICA commands
			
			rm -rf ${a2f_Out}
			# Create a condition specific background image to facilitate component rejection. Made by registering MNI image to functional image.
			if [ ! -f ${a2f_Out} ]; then
				#echo "flirt -in ${BET} -ref ${Preproc} -out ${a2f_Out} -dof 6" 								>> job
				echo "flirt -in ${BET} -ref ${a2f_Ref} -applyxfm -init ${a2f_Init} -out ${a2f_Out}" >> job
			fi
			
			# Perform ICA			
			echo -n "melodic -i ${Preproc} -o ${ICA} --dimest=${dimestVALUE} -d ${dimensionalityVALUE} " 		>> job
			echo -n "--tr=${TR} --report --guireport=${Report} --nobet --bgthreshold=${bgthresholdVALUE} " 	>> job
			echo "--mmthresh=${mmthreshVALUE} --bgimage=${a2f_Out} ${AdditionalParameters}" 					>> job

			# Error Log
			echo "cd ${FuncPath}/FEAT.feat/filtered_func_data.ica" 												>> job
			echo "Log=\`grep \"finished!\" log.txt | tail -1\`" 												>> job
			echo "if [ ! \"\$Log\" == \"finished!\" ]; then echo 'Error in ${FuncImage}' >> ${Error_Log}; fi"	>> job
						
			qsub job
			rm job
			
		done
	done
done
